#!/usr/bin/env perl
use strict;

print STDERR "\nSubmarine App Signer v1.2. (c) 2009, Deep IT.\n\n";

# 0 - App Bundle
# 1 - Output Zip file
# 2 - Output Version file

die "Invalid arguments count.\nUsage: $0 <AppName.app> <AppName.zip> <http://url/AppName.zip> <AppName.version> <minOSVersion>\n" if (scalar(@ARGV) != 5);

sub doEscape
{
	my $str = shift;
 my $esc = ' !&"\'';
	$str =~ s/([$esc])/\\$1/gos;
	return $str;
}

my ($app, $zip, $url, $ver, $minos) = map { doEscape($_) } @ARGV;
$url = $ARGV[2];

sub writeFile
{
	my ($fileName, $data) = @_;

	local * O;
	open(O,">$fileName") or die "Can't write to file $fileName\n";
	print O $data;
	close(O);
}

sub readFile
{
	my $fileName = shift;

	undef $/;
	local * F;
	open(F,"<$fileName") or die "Can't read file $fileName\n";
	my $content = <F>;
	close(F);
	return $content;
}

print STDERR "App: $app\n";
print STDERR "Zip: $zip\n";
print STDERR "URL: $url\n";
print STDERR "Ver: $ver\n";

die "Invalid arguments.\n" if ($ver eq '' or $app eq $zip or $zip eq $ver or $app eq $ver or $app !~ /\.app$/ or $zip !~ /\.zip$/ or $url eq '');

my $sign = '';
sub signIntegrity
{
	my ($dir, $rel) = @_;

	local * D;
	opendir(D,$dir) or die "Can't open directory $dir\n";
	my $item;
	while ($item = readdir(D)) 
	{
		next if ($item =~ /^\./so);
		my $full = $dir . '/' . $item;
		my $newRel = $rel . '/' . $item;
		if (-d $full)
		{
			signIntegrity($full, $newRel);
		}
		else
		{
			if (-f $full && ! -l $full && ! -b $full && ! -c $full && ! -p $full && ! -S $full)
			{
				my @stat = stat($full);
				if ($full !~ /=/ && scalar(@stat) && $stat[7])
				{
					$full = doEscape($full);
					my $md5 = `openssl dgst -md5 < "$full"`;
					$md5 =~ s/\s+$//gos;
					if ($md5)
					{
						$sign .= "$newRel=$md5\n";
					}
				}
			}
		}
	}
	closedir(D);
}

my $itg = $app;
$itg =~ s/\/[^\/]+$//gs;
$itg .= '/Contents/Submarine.integrity';

system("rm $itg 2>/dev/null");

my $plist = $app;
$plist =~ s/\/[^\/]+$//gs;
$plist .= '/Contents/Info.plist';

print STDERR "Plist file: $plist\n";
$plist = readFile($plist);

signIntegrity($app);

my $build;
if ($plist =~ /<key>CFBundleVersion<\/key>.*?<string>(.*?)<\/string>/so)
{
	$build = $1;
	die "Build number: only digits allowed.\n" if ($build !~ /^\d+$/);
}
else
{
	die "Can't fetch build number from Info.plist (CFBundleVersion)\n";
}

my $version;
if ($plist =~ /<key>CFBundleShortVersionString<\/key>.*?<string>(.*?)<\/string>/so)
{
	$version = $1;
}
else
{
	die "Can't fetch version number from Info.plist (CFBundleShortVersionString)\n";
}

print STDERR "Build: $build\n";

print STDERR "Sign:\n$sign\n" if ($sign);

writeFile($itg,$sign);

system("zip -r -y \"$zip\" \"$app\"");
my $md5 = `openssl dgst -md5 < "$zip"`;
$md5 =~ s/\s+$//gos;

my @stat = stat($zip);
die "Can't stat zip file $zip." if (!scalar(@stat));

my $length = $stat[7];

my $verTemplate = "build=$build\nurl=$url\nlength=$length\nmd5=$md5\nversion=$version\nminos=$minos";
writeFile($ver, $verTemplate);
